#!/usr/bin/env python2
##################################################
# GNU Radio Python Flow Graph
# Title: Gr FM RDS
# Generated: Wed Oct 14 20:09:00 2015
##################################################

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print "Warning: failed to XInitThreads()"

from PyQt4 import Qt
from gnuradio import analog
from gnuradio import audio
from gnuradio import blocks
from gnuradio import digital
from gnuradio import eng_notation
from gnuradio import filter
from gnuradio import gr
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from gnuradio.qtgui import Range, RangeWidget
from math import pi
from optparse import OptionParser
import osmosdr
import rds
import sip
import sys
import time

from distutils.version import StrictVersion
class gr_fm_rds(gr.top_block, Qt.QWidget):

    def __init__(self):
        gr.top_block.__init__(self, "Gr FM RDS")
        Qt.QWidget.__init__(self)
        self.setWindowTitle("Gr FM RDS")
        try:
             self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
        except:
             pass
        self.top_scroll_layout = Qt.QVBoxLayout()
        self.setLayout(self.top_scroll_layout)
        self.top_scroll = Qt.QScrollArea()
        self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
        self.top_scroll_layout.addWidget(self.top_scroll)
        self.top_scroll.setWidgetResizable(True)
        self.top_widget = Qt.QWidget()
        self.top_scroll.setWidget(self.top_widget)
        self.top_layout = Qt.QVBoxLayout(self.top_widget)
        self.top_grid_layout = Qt.QGridLayout()
        self.top_layout.addLayout(self.top_grid_layout)

        self.settings = Qt.QSettings("GNU Radio", "gr_fm_rds")
        self.restoreGeometry(self.settings.value("geometry").toByteArray())


        ##################################################
        # Variables
        ##################################################
        self.vol = vol = 5.0
        self.tune_freq = tune_freq = 93.80e6
        self.samp_rate = samp_rate = 256000

        ##################################################
        # Blocks
        ##################################################
        self._vol_range = Range(0, 50.0, 1.0, 5.0, 200)
        self._vol_win = RangeWidget(self._vol_range, self.set_vol, "Volume", "counter_slider")
        self.top_grid_layout.addWidget(self._vol_win, 2,1)
        self._tune_freq_tool_bar = Qt.QToolBar(self)
        self._tune_freq_tool_bar.addWidget(Qt.QLabel("Frequenz"+": "))
        self._tune_freq_line_edit = Qt.QLineEdit(str(self.tune_freq))
        self._tune_freq_tool_bar.addWidget(self._tune_freq_line_edit)
        self._tune_freq_line_edit.returnPressed.connect(
        	lambda: self.set_tune_freq(eng_notation.str_to_num(str(self._tune_freq_line_edit.text().toAscii()))))
        self.top_grid_layout.addWidget(self._tune_freq_tool_bar, 1,1)
        self.rtlsdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + "" )
        self.rtlsdr_source_0.set_sample_rate(samp_rate*4)
        self.rtlsdr_source_0.set_center_freq(tune_freq, 0)
        self.rtlsdr_source_0.set_freq_corr(38, 0)
        self.rtlsdr_source_0.set_dc_offset_mode(0, 0)
        self.rtlsdr_source_0.set_iq_balance_mode(0, 0)
        self.rtlsdr_source_0.set_gain_mode(True, 0)
        self.rtlsdr_source_0.set_gain(20, 0)
        self.rtlsdr_source_0.set_if_gain(20, 0)
        self.rtlsdr_source_0.set_bb_gain(20, 0)
        self.rtlsdr_source_0.set_antenna("", 0)
        self.rtlsdr_source_0.set_bandwidth(0, 0)
          
        self.root_raised_cosine_filter_0 = filter.fir_filter_fff(20, firdes.root_raised_cosine(
        	1, samp_rate, 2375, 1, 100))
        self.rds_display_0 = rds.display()
        self._rds_display_0_win = sip.wrapinstance(self.rds_display_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._rds_display_0_win, 3,1)
        self.low_pass_filter_0 = filter.fir_filter_fff(8, firdes.low_pass(
        	1, samp_rate, 15000, 5000, firdes.WIN_HAMMING, 6.76))
        self.digital_diff_decoder_bb_0 = digital.diff_decoder_bb(2)
        self.digital_clock_recovery_mm_xx_0_0 = digital.clock_recovery_mm_ff(samp_rate / 20 / (1187.5 * 2), 0.01, 0.5, 0.01, 0.1)
        self.digital_binary_slicer_fb_0 = digital.binary_slicer_fb()
        self.blocks_null_source_0 = blocks.null_source(gr.sizeof_float*1)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_vff((vol, ))
        self.blocks_keep_one_in_n_0 = blocks.keep_one_in_n(gr.sizeof_char*1, 2)
        self.blocks_float_to_complex_0 = blocks.float_to_complex(1)
        self.blocks_complex_to_real_0 = blocks.complex_to_real(1)
        self.audio_sink_0 = audio.sink(32000, "", True)
        self.analog_wfm_rcv_0 = analog.wfm_rcv(
        	quad_rate=samp_rate*4,
        	audio_decimation=4,
        )
        self.analog_pll_carriertracking_cc_0_0 = analog.pll_carriertracking_cc(1e-3, 1.0001*2*pi*57000/samp_rate, 0.9999*2*pi*57000/samp_rate)
        self.analog_fm_deemph_0 = analog.fm_deemph(fs=samp_rate/8, tau=75e-6)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.analog_fm_deemph_0, 0), (self.blocks_multiply_const_vxx_0, 0))    
        self.connect((self.analog_pll_carriertracking_cc_0_0, 0), (self.blocks_complex_to_real_0, 0))    
        self.connect((self.analog_wfm_rcv_0, 0), (self.blocks_float_to_complex_0, 0))    
        self.connect((self.analog_wfm_rcv_0, 0), (self.low_pass_filter_0, 0))    
        self.connect((self.blocks_complex_to_real_0, 0), (self.root_raised_cosine_filter_0, 0))    
        self.connect((self.blocks_float_to_complex_0, 0), (self.analog_pll_carriertracking_cc_0_0, 0))    
        self.connect((self.blocks_keep_one_in_n_0, 0), (self.digital_diff_decoder_bb_0, 0))    
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.audio_sink_0, 0))    
        self.connect((self.blocks_null_source_0, 0), (self.blocks_float_to_complex_0, 1))    
        self.connect((self.digital_binary_slicer_fb_0, 0), (self.blocks_keep_one_in_n_0, 0))    
        self.connect((self.digital_clock_recovery_mm_xx_0_0, 0), (self.digital_binary_slicer_fb_0, 0))    
        self.connect((self.digital_diff_decoder_bb_0, 0), (self.rds_display_0, 0))    
        self.connect((self.low_pass_filter_0, 0), (self.analog_fm_deemph_0, 0))    
        self.connect((self.root_raised_cosine_filter_0, 0), (self.digital_clock_recovery_mm_xx_0_0, 0))    
        self.connect((self.rtlsdr_source_0, 0), (self.analog_wfm_rcv_0, 0))    

    def closeEvent(self, event):
        self.settings = Qt.QSettings("GNU Radio", "gr_fm_rds")
        self.settings.setValue("geometry", self.saveGeometry())
        event.accept()

    def get_vol(self):
        return self.vol

    def set_vol(self, vol):
        self.vol = vol
        self.blocks_multiply_const_vxx_0.set_k((self.vol, ))

    def get_tune_freq(self):
        return self.tune_freq

    def set_tune_freq(self, tune_freq):
        self.tune_freq = tune_freq
        Qt.QMetaObject.invokeMethod(self._tune_freq_line_edit, "setText", Qt.Q_ARG("QString", eng_notation.num_to_str(self.tune_freq)))
        self.rtlsdr_source_0.set_center_freq(self.tune_freq, 0)

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.rtlsdr_source_0.set_sample_rate(self.samp_rate*4)
        self.analog_pll_carriertracking_cc_0_0.set_max_freq(1.0001*2*pi*57000/self.samp_rate)
        self.analog_pll_carriertracking_cc_0_0.set_min_freq(0.9999*2*pi*57000/self.samp_rate)
        self.root_raised_cosine_filter_0.set_taps(firdes.root_raised_cosine(1, self.samp_rate, 2375, 1, 100))
        self.digital_clock_recovery_mm_xx_0_0.set_omega(self.samp_rate / 20 / (1187.5 * 2))
        self.low_pass_filter_0.set_taps(firdes.low_pass(1, self.samp_rate, 15000, 5000, firdes.WIN_HAMMING, 6.76))


if __name__ == '__main__':
    parser = OptionParser(option_class=eng_option, usage="%prog: [options]")
    (options, args) = parser.parse_args()
    if(StrictVersion(Qt.qVersion()) >= StrictVersion("4.5.0")):
        Qt.QApplication.setGraphicsSystem(gr.prefs().get_string('qtgui','style','raster'))
    qapp = Qt.QApplication(sys.argv)
    tb = gr_fm_rds()
    tb.start()
    tb.show()
    def quitting():
        tb.stop()
        tb.wait()
    qapp.connect(qapp, Qt.SIGNAL("aboutToQuit()"), quitting)
    qapp.exec_()
    tb = None #to clean up Qt widgets
