/* -*- c++ -*- */
/* 
 * Copyright 2015 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_RDS_DISPLAY_H
#define INCLUDED_RDS_DISPLAY_H

#ifdef ENABLE_PYTHON
#include <Python.h>
#endif

#include <rds/api.h>
#include <gnuradio/sync_block.h>

#include <qapplication.h>

namespace gr {
  namespace rds {

    /*!
     * \brief <+description of block+>
     * \ingroup rds
     *
     */
    class RDS_API display : virtual public gr::sync_block
    {
     public:
      typedef boost::shared_ptr<display> sptr;

      /*!
       * \brief Return a shared_ptr to a new instance of rds::display.
       *
       * To avoid accidental use of raw pointers, rds::display's
       * constructor is in a private implementation
       * class. rds::display::make is the public interface for
       * creating new instances.
       */
      static sptr make(QWidget *parent=NULL);

      virtual void exec_() = 0;
      virtual QWidget* qwidget() = 0;

#ifdef ENABLE_PYTHON
      virtual PyObject* pyqwidget() = 0;
#else
      virtual void* pyqwidget() = 0;
#endif

      QApplication *d_qApplication;
    };

  } // namespace rds
} // namespace gr

#endif /* INCLUDED_RDS_DISPLAY_H */

