#include "rdswidget.h"
#include "ui_rdswidget.h"

#include "EBUtab1.h"

RDSWidget::RDSWidget(QWidget *parent)
: QWidget(parent), ui(new Ui::RDSWidget)
{
   ui->setupUi(this);
}

RDSWidget::~RDSWidget()
{
   delete ui;
}

void RDSWidget::customEvent(QEvent *e)
{
   RDSUpdateEvent *RDSe = dynamic_cast<RDSUpdateEvent *>(e);
   if ( ! RDSe ) return;

   ui->l_blkerr->setText( QString::number(RDSe->blkerr*100.0,'f',1) );

   QString Fstr;
   if ( RDSe->D.TP ) { Fstr += "TP, "; }
   if ( RDSe->D.TA ) { Fstr += "TA, "; }
   if ( RDSe->D.MS ) { Fstr += "MS, "; }
   Fstr += ( RDSe->D.DI[3] ? "stereo, " : "mono, " );
   if ( RDSe->D.DI[2] ) { Fstr += "AH, "; }
   if ( RDSe->D.DI[1] ) { Fstr += "compressed, "; }
   Fstr += ( RDSe->D.DI[0] ? "dynamic PTY" : "static PTY" );
   ui->l_flags->setText( Fstr );

   QString PS;
   for ( unsigned int i=0; i<8; ++i ) { PS.append( EBUtab1[RDSe->D.PS[i]] ); }
   ui->l_PS->setText( PS );

   ui->l_PTY->setText( QString::number(RDSe->D.PTY) );

   QString RT;
   for ( unsigned int i=0; i<64; ++i ) { RT.append( EBUtab1[RDSe->D.RT[i]] ); }
   ui->l_RT->setText( RT );

   unsigned int MJD = RDSe->D.timeJD;
   int Ys = (int)( (MJD - 15078.2)/365.25 );
   int Ms = (int)( (MJD - 14956.1 - (int)(Ys*365.25))/30.6001 );
   int D = MJD - 14956 - (int)(Ys*365.25) - (int)(Ms*30.6001);
   int K = ( Ms==14 || Ms==15 ? 1 : 0 );
   int Y = Ys + K + 1900;
   int M = Ms - 1 - K*12;
   ui->l_MJD->setText( QString("%1:%2-%3-%4").arg(MJD).arg(Y).arg(M,2,10,QChar('0')).arg(D,2,10,QChar('0')) );
   ui->l_UTC->setText( QString::number(RDSe->D.timeHour)+":"+QString("%1").arg(RDSe->D.timeMinute,2,10,QChar('0')) );
   ui->l_TZ->setText( QString::number(RDSe->D.timeOffset*0.5,'f',1) );

   QString AFstr;
   for ( unsigned int i=0; i<RDSe->D.AF.no; ++i )
   {
       AFstr += QString::number(RDSe->D.AF.kHz[i]) + "kHz ";
   }
   ui->l_AF->setText( AFstr );

}
