##################################################
# GNU Radio Python Flow Graph
# Title: RDS Demodulator
# Generated: Tue Oct 13 19:20:55 2015
##################################################

from gnuradio import blocks
from gnuradio import digital
from gnuradio import filter
from gnuradio import gr
from gnuradio.filter import firdes
import rds

class rds_demod(gr.hier_block2):

    def __init__(self, samp_rate=285000):
        gr.hier_block2.__init__(
            self, "RDS Demodulator",
            gr.io_signature(1, 1, gr.sizeof_float*1),
            gr.io_signature(1, 1, gr.sizeof_char*1),
        )

        ##################################################
        # Parameters
        ##################################################
        self.samp_rate = samp_rate

        ##################################################
        # Blocks
        ##################################################
        self.rds_clock_0 = rds.clock(samp_rate)
        self.low_pass_filter_0 = filter.fir_filter_fff(10, firdes.low_pass(
        	1, samp_rate, 1200, 2400, firdes.WIN_HAMMING, 6.76))
        self.digital_diff_decoder_bb_0 = digital.diff_decoder_bb(2)
        self.digital_clock_recovery_mm_xx_0 = digital.clock_recovery_mm_ff(samp_rate/(1187.5 * 10), 0.01, 0.5, 0.01, 0.1)
        self.digital_binary_slicer_fb_0 = digital.binary_slicer_fb()
        self.blocks_multiply_xx_0 = blocks.multiply_vff(1)
        self.blocks_complex_to_real_0 = blocks.complex_to_real(1)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_complex_to_real_0, 0), (self.blocks_multiply_xx_0, 0))    
        self.connect((self.blocks_multiply_xx_0, 0), (self.low_pass_filter_0, 0))    
        self.connect((self.digital_binary_slicer_fb_0, 0), (self.digital_diff_decoder_bb_0, 0))    
        self.connect((self.digital_clock_recovery_mm_xx_0, 0), (self.digital_binary_slicer_fb_0, 0))    
        self.connect((self.digital_diff_decoder_bb_0, 0), (self, 0))    
        self.connect((self.low_pass_filter_0, 0), (self.digital_clock_recovery_mm_xx_0, 0))    
        self.connect((self, 0), (self.rds_clock_0, 0))    
        self.connect((self.rds_clock_0, 0), (self.blocks_complex_to_real_0, 0))    
        self.connect((self.rds_clock_0, 1), (self.blocks_multiply_xx_0, 1))    


    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.low_pass_filter_0.set_taps(firdes.low_pass(1, self.samp_rate, 1200, 2400, firdes.WIN_HAMMING, 6.76))
        self.digital_clock_recovery_mm_xx_0.set_omega(self.samp_rate/(1187.5 * 10))

